@@ -28,7 +28,7 @@
 
 namespace ExifOperations
 {
-	void writeExifData(const std::string& filename, const std::string& comment);
+	void writeExifData(const std::string& filename, const std::string& comment, float expotime = 100.0f);
 	void copyExifData(const std::string& from, const std::string& to, bool dont_overwrite);
 	float obtain_avg_lum(const std::string& filename);
 	int obtain_rotation(const std::string& filename);
