@@ -27,11 +27,15 @@
 #define FROMLDRPFSTO16BITSPIXMAP
 
 #include <QtGlobal>
+#include <Common/FloatRgbToQRgb.h>
 
 namespace pfs {
     class Frame;
 }
 
-quint16* fromLDRPFSto16bitsPixmap(pfs::Frame* inpfsframe, float min_luminance = 0.0f, float max_luminance = 1.0f);
+quint16* fromLDRPFSto16bitsPixmap(pfs::Frame* inpfsframe,
+                                  float min_luminance = 0.0f,
+                                  float max_luminance = 1.0f,
+                                  LumMappingMethod mapping_method = MAP_LINEAR);
 
 #endif
