@@ -51,8 +51,11 @@ namespace
 void printErrorAndExit(const QString& error_str)
 {
 	#if defined(_MSC_VER)
-		_setmode(_fileno(stderr), _O_U16TEXT);
+		// if the filemode isn't restored afterwards, a normal std::cout segfaults
+		int oldMode = _setmode(_fileno(stderr), _O_U16TEXT);
 		std::wcerr << qPrintable(error_str) << std::endl;
+		if (oldMode >= 0)
+			_setmode(_fileno(stderr), oldMode);
 	#else
 		std::cerr << qPrintable(error_str) << std::endl;
 	#endif
@@ -64,8 +67,11 @@ void printIfVerbose(const QString& str, bool verbose)
     if ( verbose )
     {
 		#if defined(_MSC_VER)
-			_setmode(_fileno(stdout), _O_U16TEXT);
+    		// if the filemode isn't restored afterwards, a normal std::cout segfaults
+			int oldMode = _setmode(_fileno(stdout), _O_U16TEXT);
 			std::wcout << qPrintable(str) << std::endl;
+			if (oldMode >= 0)
+				_setmode(_fileno(stdout), oldMode);
 		#else
     		std::cout << qPrintable(str) << std::endl;
 		#endif
@@ -140,6 +146,7 @@ static struct option cmdLineOptions[] = {
     { "tmo", required_argument, NULL, 't' },
     { "tmoptions", required_argument, NULL, 'p' },
     { "output", required_argument, NULL, 'o' },
+    { "quality", required_argument, NULL, 'q' },
     { NULL, 0, NULL, 0 }
 };
 
@@ -149,7 +156,8 @@ CommandLineInterfaceManager::CommandLineInterfaceManager(const int argc, char **
     operationMode(UNKNOWN_MODE),
     alignMode(NO_ALIGN),
     tmopts(TMOptionsOperations::getDefaultTMOptions()),
-    verbose(false)
+    verbose(false),
+	quality(100)
 {
     hdrcreationconfig.weights = TRIANGULAR;
     hdrcreationconfig.response_curve = LINEAR;
@@ -169,7 +177,7 @@ void CommandLineInterfaceManager::parseArgs()
 //        cliOptions += cmdLineOptions[i].val;
 //    }
 
-    while( (c = getopt_long_only (argc, argv, ":hva:e:c:l:s:g:r:t:p:o:u", cmdLineOptions, &optionIndex)) != -1 )
+    while( (c = getopt_long_only (argc, argv, ":hva:e:c:l:s:g:r:t:p:o:u:q:", cmdLineOptions, &optionIndex)) != -1 )
     {
         switch ( c )
         {
@@ -398,6 +406,11 @@ void CommandLineInterfaceManager::parseArgs()
         case 'o':
             saveLdrFilename=QString(optarg);
             break;
+        case 'q':
+            quality = QString(optarg).toInt();
+			if (quality < 0 || quality > 100)
+            	printErrorAndExit(tr("Error: Quality must be in the range [0-100]."));
+            break;
         case '?':
             printErrorAndExit(tr("Error: Unknown option %1.").arg(optopt));
         case ':':
@@ -451,10 +464,15 @@ void CommandLineInterfaceManager::execCommandLineParamsSlot()
             printIfVerbose(QObject::tr("Using %1 threads.").arg(luminance_options.getNumThreads()), verbose);
         }
 
-        hdrCreationManager.reset( new HdrCreationManager() );
+        hdrCreationManager.reset( new HdrCreationManager(true) );
         connect(hdrCreationManager.data(), SIGNAL(finishedLoadingInputFiles(QStringList)), this, SLOT(finishedLoadingInputFiles(QStringList)));
-        connect(hdrCreationManager.data(), SIGNAL(finishedAligning()), this, SLOT(createHDR()));
+        connect(hdrCreationManager.data(), SIGNAL(finishedAligning(int)), this, SLOT(createHDR(int)));
         connect(hdrCreationManager.data(), SIGNAL(ais_failed(QProcess::ProcessError)), this, SLOT(ais_failed(QProcess::ProcessError)));
+		connect(hdrCreationManager.data(), SIGNAL(errorWhileLoading(QString)),this, SLOT(errorWhileLoading(QString)));
+		//connect(hdrCreationManager.data(), SIGNAL(maximumValue(int)),this, SLOT(setProgressBar(int)));
+		//connect(hdrCreationManager.data(), SIGNAL(nextstep(int)),this, SLOT(updateProgressBar(int)));
+		connect(hdrCreationManager.data(), SIGNAL(aisDataReady(QByteArray)),this, SLOT(readData(QByteArray)));
+			
         hdrCreationManager->setConfig(hdrcreationconfig);
         hdrCreationManager->setFileList(inputFiles);
         hdrCreationManager->loadInputFiles();
@@ -481,7 +499,7 @@ void CommandLineInterfaceManager::finishedLoadingInputFiles(QStringList filesLac
 {
     if (filesLackingExif.size()!=0 && ev.isEmpty())
     {
-        printErrorAndExit(tr("Error: Exif data missing in images and EV values not specifed on the commandline, bailing out."));
+        printErrorAndExit(tr("Error: Exif data missing in images and EV values not specified on the commandline, bailing out."));
     }
     if (!ev.isEmpty())
     {
@@ -501,7 +519,7 @@ void CommandLineInterfaceManager::finishedLoadingInputFiles(QStringList filesLac
     }
     else if (alignMode == NO_ALIGN)
     {
-        createHDR();
+        createHDR(0);
     }
 }
 
@@ -510,8 +528,10 @@ void CommandLineInterfaceManager::ais_failed(QProcess::ProcessError)
     printErrorAndExit( tr("Failed executing align_image_stack"));
 }
 
-void CommandLineInterfaceManager::createHDR()
+void CommandLineInterfaceManager::createHDR(int errorcode)
 {
+	if (errorcode != 0)
+		printIfVerbose( tr("Failed aligning images.") , verbose);
     hdrCreationManager->removeTempFiles();
 
     printIfVerbose( tr("Creating (in memory) the HDR.") , verbose);
@@ -560,13 +580,15 @@ void  CommandLineInterfaceManager::startTonemap()
 
         // Build TMWorker
         TMWorker tm_worker;
+		connect(&tm_worker, SIGNAL(tonemapSetMaximum(int)), this, SLOT(setProgressBar(int)));
+		connect(&tm_worker, SIGNAL(tonemapSetValue(int)), this, SLOT(updateProgressBar(int)));
 
         // Build a new TM frame
         // The scoped pointer will free the memory automatically later on
         QScopedPointer<pfs::Frame> tm_frame( tm_worker.computeTonemap(HDR.data(), tmopts.data()) );
 
         // Create an ad-hoc IOWorker to save the file
-        if ( IOWorker().write_ldr_frame(tm_frame.data(), saveLdrFilename, 100, tmopts.data()) )
+        if ( IOWorker().write_ldr_frame(tm_frame.data(), saveLdrFilename, quality, tmopts.data()) )
         {
             // File save successful
             printIfVerbose( tr("Image %1 saved successfully").arg(saveLdrFilename) , verbose);
@@ -586,7 +608,9 @@ void  CommandLineInterfaceManager::startTonemap()
     }
 }
 
-
+void CommandLineInterfaceManager::errorWhileLoading(QString errormessage) {
+	printErrorAndExit( tr("Failed loading images"));
+}
 
 void CommandLineInterfaceManager::printHelp(char * progname)
 {
@@ -619,7 +643,48 @@ void CommandLineInterfaceManager::printHelp(char * progname)
             "\t\t" + tr("brightness=VALUE:chroma=VALUE:lightness=VALUE (for reinhard05)") + "\n" +
             "\t\t" + tr("(default is contrast=0.3:equalization=false:saturation=1.8, see also -o)") + "\n" +
             "\t" + tr("-o --output LDR_FILE   File name you want to save your tone mapped LDR to.") + "\n" +
+            "\t" + tr("-q --quality VALUE   Quality of the saved tone mapped file (0-100).") + "\n" +
             "\t" + tr("                       (No tonemapping is performed unless -o is specified).") + "\n\n" +
             tr("You must either load an existing HDR file (via the -l option) or specify INPUTFILES to create a new HDR.\n");
     printErrorAndExit(help);
 }
+
+void CommandLineInterfaceManager::setProgressBar(int max)
+{
+	maximum = max;
+	progressBar.reset();
+	progressBar.n = max;
+	std::cout << std::endl;
+	started = true;
+}
+
+void CommandLineInterfaceManager::updateProgressBar(int value)
+{
+	if (verbose) {
+		if (value < 0) return;
+		if (value < oldValue) {
+			//progressBar.reset();
+			//progressBar.n = maximum;
+			//progressBar.start();
+			progressBar.cur = value;
+			progressBar.setPct( ((float)value)/maximum );
+		}
+		if (started) {
+			started = false;
+			progressBar.start();
+		}
+		for (int i = 0; i < value - oldValue; i++)
+			++progressBar;
+		oldValue = value;
+		//if (value == progressBar.n) {
+		//	std::cout << std::endl;
+		//}
+	}
+}
+
+void CommandLineInterfaceManager::readData(QByteArray data)
+{
+	if (verbose)
+		std::cout << data.constData() << std::endl;
+}
+
