@@ -55,6 +55,7 @@
 #include "Common/archs.h"
 #include "Common/config.h"
 #include "Common/global.h"
+#include "OsIntegration/osintegration.h"
 #include "TonemappingPanel/TonemappingWarnDialog.h"
 #include "BatchHDR/BatchHDRDialog.h"
 #include "BatchTM/BatchTMDialog.h"
@@ -153,7 +154,6 @@ MainWindow::MainWindow(QWidget *parent):
 MainWindow::MainWindow(pfs::Frame* curr_frame, QString new_file, bool needSaving, QWidget *parent):
     QMainWindow(parent),
     m_Ui(new Ui::MainWindow)
-
 {
     init();
 
@@ -186,6 +186,8 @@ MainWindow::~MainWindow()
 
 void MainWindow::init()
 {
+    OsIntegration::getInstance().init(this);
+
     sm_NumMainWindows++;
 
     helpBrowser = NULL;
@@ -280,6 +282,7 @@ void MainWindow::createCentralWidget()
 
     connect(m_tabwidget, SIGNAL(tabCloseRequested(int)), this, SLOT(removeTab(int)));
     connect(m_tabwidget, SIGNAL(currentChanged(int)), this, SLOT(updateActions(int)));
+    connect(m_tabwidget, SIGNAL(currentChanged(int)), this, SLOT(updateSoftProofing(int)));
     connect(tmPanel, SIGNAL(startTonemapping(TonemappingOptions*)), this, SLOT(tonemapImage(TonemappingOptions*)));
     connect(this, SIGNAL(updatedHDR(pfs::Frame*)), tmPanel, SLOT(updatedHDR(pfs::Frame*)));
     connect(this, SIGNAL(destroyed()), previewPanel, SLOT(deleteLater()));
@@ -310,7 +313,6 @@ void MainWindow::createToolBar()
     toolBarOptsGroup->addAction(m_Ui->actionText_Only);
     m_Ui->menuToolbars->addAction(m_Ui->toolBar->toggleViewAction());
 
-    connect(m_Ui->actionLock, SIGNAL(toggled(bool)), this, SLOT(lockViewers(bool)));
     connect(m_Ui->actionText_Under_Icons,SIGNAL(triggered()),this,SLOT(Text_Under_Icons()));
     connect(m_Ui->actionIcons_Only,SIGNAL(triggered()),this,SLOT(Icons_Only()));
     connect(m_Ui->actionText_Alongside_Icons,SIGNAL(triggered()),this,SLOT(Text_Alongside_Icons()));
@@ -321,61 +323,23 @@ void MainWindow::createMenus()
 {
     // About(s)
     connect(m_Ui->actionAbout_Qt,SIGNAL(triggered()),qApp,SLOT(aboutQt()));
-    connect(m_Ui->actionAbout_Luminance,SIGNAL(triggered()),this,SLOT(aboutLuminance()));
-    connect(m_Ui->actionDonate, SIGNAL(activated()), this, SLOT(showDonationsPage()));
 
-    connect(m_Ui->OptionsAction,SIGNAL(triggered()),this,SLOT(preferences_called()));
-    connect(m_Ui->documentationAction,SIGNAL(triggered()),this,SLOT(openDocumentation()));
     connect(m_Ui->actionWhat_s_This,SIGNAL(triggered()),this,SLOT(enterWhatsThis()));
 
     // I/O
-    connect(m_Ui->fileNewAction, SIGNAL(triggered()), this, SLOT(fileNewViaWizard()));
-    connect(m_Ui->fileOpenAction, SIGNAL(triggered()), this, SLOT(fileOpen()));
-    connect(m_Ui->fileSaveAsAction, SIGNAL(triggered()), this, SLOT(fileSaveAs()));
-    connect(m_Ui->fileSaveAllAction, SIGNAL(triggered()), this, SLOT(fileSaveAll()));
     connect(m_Ui->fileExitAction, SIGNAL(triggered()), this, SLOT(close()));
-    connect(m_Ui->actionSave_Hdr_Preview, SIGNAL(triggered()), this, SLOT(saveHdrPreview()));
-
-    // HDR Editing
-    connect(m_Ui->actionResizeHDR, SIGNAL(triggered()), this, SLOT(resize_requested()));
-    connect(m_Ui->action_Projective_Transformation, SIGNAL(triggered()), this, SLOT(projectiveTransf_requested()));
-
-    // HDR Exposure Control
-    connect(m_Ui->Low_dynamic_range,SIGNAL(triggered()),this,SLOT(hdr_ldr_exp()));
-    connect(m_Ui->Fit_to_dynamic_range,SIGNAL(triggered()),this,SLOT(hdr_fit_exp()));
-    connect(m_Ui->Shrink_dynamic_range,SIGNAL(triggered()),this,SLOT(hdr_shrink_exp()));
-    connect(m_Ui->Extend_dynamic_range,SIGNAL(triggered()),this,SLOT(hdr_extend_exp()));
-    connect(m_Ui->Decrease_exposure,SIGNAL(triggered()),this,SLOT(hdr_decrease_exp()));
-    connect(m_Ui->Increase_exposure,SIGNAL(triggered()),this,SLOT(hdr_increase_exp()));
 
     // Crop & Rotation
     connect(m_Ui->cropToSelectionAction, SIGNAL(triggered()), this, SLOT(cropToSelection()));
     m_Ui->cropToSelectionAction->setEnabled(false);
 
     connect(m_Ui->removeSelectionAction, SIGNAL(triggered()), this, SLOT(disableCrop()));
-    connect(m_Ui->rotateccw, SIGNAL(triggered()), this, SLOT(rotateccw_requested()));
-    connect(m_Ui->rotatecw, SIGNAL(triggered()), this, SLOT(rotatecw_requested()));
-
-    // Zoom
-    connect(m_Ui->zoomInAct,SIGNAL(triggered()),this,SLOT(viewerZoomIn()));
-    connect(m_Ui->zoomOutAct,SIGNAL(triggered()),this,SLOT(viewerZoomOut()));
-    connect(m_Ui->fitToWindowAct,SIGNAL(triggered()),this,SLOT(viewerFitToWin()));
-    connect(m_Ui->actionFill_to_Window,SIGNAL(triggered()),this,SLOT(viewerFillToWin()));
-    connect(m_Ui->normalSizeAct,SIGNAL(triggered()),this,SLOT(viewerOriginalSize()));
-
-    // Tools
-    connect(m_Ui->Transplant_Exif_Data_action,SIGNAL(triggered()),this,SLOT(transplant_called()));
-    connect(m_Ui->actionBatch_HDR, SIGNAL(triggered()), this, SLOT(batch_hdr_requested()));
-    connect(m_Ui->actionBatch_Tone_Mapping, SIGNAL(triggered()), this, SLOT(batch_requested()));
 
     connect(m_Ui->menuWindows, SIGNAL(aboutToShow()), this, SLOT(updateWindowMenu()));
-    connect(m_Ui->actionMinimize, SIGNAL(triggered()), this, SLOT(minimizeMW()));
-    connect(m_Ui->actionMaximize, SIGNAL(triggered()), this, SLOT(maximizeMW()));
-    connect(m_Ui->actionBring_All_to_Front, SIGNAL(triggered()), this, SLOT(bringAllMWToFront()));
+    connect(m_Ui->actionMinimize, SIGNAL(triggered()), this, SLOT(showMinimized()));
+    connect(m_Ui->actionMaximize, SIGNAL(triggered()), this, SLOT(showMaximized()));
     connect(m_Ui->actionShowPreviewPanel, SIGNAL(toggled(bool)), this, SLOT(showPreviewPanel(bool)));
     connect(m_Ui->actionShowPreviewPanel, SIGNAL(toggled(bool)), &luminance_options, SLOT(setPreviewPanelActive(bool)));
-    connect(m_Ui->actionFix_Histogram,SIGNAL(toggled(bool)),this,SLOT(levelsRequested(bool)));
-    connect(m_Ui->actionRemove_Tab,SIGNAL(triggered()),this,SLOT(removeCurrentTab()));
 
     //recent files
     initRecentFileActions();
@@ -389,9 +353,6 @@ void MainWindow::createStatusBar()
 
 void MainWindow::createConnections()
 {
-    connect(m_Ui->actionShowNext, SIGNAL(triggered()), this, SLOT(activateNextViewer()));
-    connect(m_Ui->actionShowPrevious, SIGNAL(triggered()), this, SLOT(activatePreviousViewer()));
-
     windowMapper = new QSignalMapper(this);
     connect(windowMapper, SIGNAL(mapped(QWidget*)), this, SLOT(setActiveMainWindow(QWidget*)));
 }
@@ -423,12 +384,17 @@ void MainWindow::loadOptions()
 
 }
 
-void MainWindow::showDonationsPage()
+void MainWindow::on_actionDonate_triggered()
 {
     QDesktopServices::openUrl(QUrl("https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=77BSTWEH7447C")); //davideanastasia
 }
 
-void MainWindow::fileNewViaWizard(QStringList files)
+void MainWindow::on_fileNewAction_triggered()
+{
+	createNewHdr(QStringList()); // redirect on createNewHdr-method to avoid moc warning
+}
+
+void MainWindow::createNewHdr(QStringList files)
 {
     QScopedPointer<HdrWizard> wizard( new HdrWizard (this, files) );
     if (wizard->exec() == QDialog::Accepted)
@@ -437,7 +403,7 @@ void MainWindow::fileNewViaWizard(QStringList files)
     }
 }
 
-void MainWindow::fileOpen()
+void MainWindow::on_fileOpenAction_triggered()
 {
     QString filetypes = tr("All HDR formats ");
     filetypes += "(*.exr *.hdr *.pic *.tiff *.tif *.pfs *.crw *.cr2 *.nef *.dng *.mrw *.orf *.kdc *.dcr *.arw *.raf *.ptx *.pef *.x3f *.raw *.rw2 *.sr2 *.3fr *.mef *.mos *.erf *.nrw *.srw";
@@ -471,7 +437,7 @@ void MainWindow::fileOpen()
     }
 }
 
-void MainWindow::fileSaveAll()
+void MainWindow::on_fileSaveAllAction_triggered()
 {
     if (m_tabwidget->count() <= 0) return;
 
@@ -503,7 +469,7 @@ void MainWindow::fileSaveAll()
     }
 }
 
-void MainWindow::fileSaveAs()
+void MainWindow::on_fileSaveAsAction_triggered()
 {
     if (m_tabwidget->count() <= 0) return;
 
@@ -610,150 +576,77 @@ void MainWindow::save_ldr_failed()
     //QMessageBox::warning(0,"",QObject::tr("Failed to save <b>") + outfname + "</b>", QMessageBox::Ok, QMessageBox::NoButton);
 }
 
-void MainWindow::saveHdrPreview()
+void MainWindow::on_actionSave_Hdr_Preview_triggered()
 {
     if (m_tabwidget->count() <= 0) return;
 
     GenericViewer* g_v = (GenericViewer*)m_tabwidget->currentWidget();
-    try {
-        HdrViewer* hdr_v = dynamic_cast<HdrViewer*>(g_v);
 
+    if (!g_v->isHDR()) return;
+    try
+    {
         QString ldr_name = QFileInfo(getCurrentHDRName()).baseName();
 
-        QString outfname = getLdrFileNameFromSaveDialog(ldr_name + "_" + hdr_v->getFileNamePostFix() + ".jpg", this);
+        QString outfname = getLdrFileNameFromSaveDialog(ldr_name + "_" + g_v->getFileNamePostFix() + ".jpg", this);
 
         if ( outfname.isEmpty() ) return;
 
         QMetaObject::invokeMethod(m_IOWorker, "write_ldr_frame", Qt::QueuedConnection,
-                                  Q_ARG(GenericViewer*, hdr_v),
+                                  Q_ARG(GenericViewer*, g_v),
                                   Q_ARG(QString, outfname),
                                   Q_ARG(int, 100));
-    } catch (...)
+    }
+    catch (...)
     {
         return;
     }
 }
 
-void MainWindow::updateActionsNoImage()
-{
-    updateMagnificationButtons(NULL);
-
-    m_Ui->fileSaveAsAction->setEnabled(false);
-    m_Ui->actionSave_Hdr_Preview->setEnabled(false);
-
-    // Histogram
-    m_Ui->menuHDR_Histogram->setEnabled(false);
-    m_Ui->Low_dynamic_range->setEnabled(false);
-    m_Ui->Fit_to_dynamic_range->setEnabled(false);
-    m_Ui->Shrink_dynamic_range->setEnabled(false);
-    m_Ui->Extend_dynamic_range->setEnabled(false);
-    m_Ui->Decrease_exposure->setEnabled(false);
-    m_Ui->Increase_exposure->setEnabled(false);
-
-    m_Ui->actionResizeHDR->setEnabled(false);
-    m_Ui->action_Projective_Transformation->setEnabled(false);
-    m_Ui->cropToSelectionAction->setEnabled(false);
-    m_Ui->rotateccw->setEnabled(false);
-    m_Ui->rotatecw->setEnabled(false);
-    m_Ui->actionFix_Histogram->setEnabled(false);
-}
-
-void MainWindow::updateActionsLdrImage()
-{
-    // Read/Save
-    m_Ui->fileSaveAsAction->setEnabled(true);
-    m_Ui->actionSave_Hdr_Preview->setEnabled(true);
-    if (curr_num_ldr_open >= 2)
-        m_Ui->fileSaveAllAction->setEnabled(true);
-
-    // Histogram
-    m_Ui->menuHDR_Histogram->setEnabled(false);
-    m_Ui->Low_dynamic_range->setEnabled(false);
-    m_Ui->Fit_to_dynamic_range->setEnabled(false);
-    m_Ui->Shrink_dynamic_range->setEnabled(false);
-    m_Ui->Extend_dynamic_range->setEnabled(false);
-    m_Ui->Decrease_exposure->setEnabled(false);
-    m_Ui->Increase_exposure->setEnabled(false);
-
-    m_Ui->actionResizeHDR->setEnabled(false);
-    m_Ui->action_Projective_Transformation->setEnabled(false);
-    m_Ui->cropToSelectionAction->setEnabled(false);
-    m_Ui->removeSelectionAction->setEnabled(false);
-    m_Ui->rotateccw->setEnabled(false);
-    m_Ui->rotatecw->setEnabled(false);
-    m_Ui->actionFix_Histogram->setEnabled(true);
-}
-
-void MainWindow::updateActionsHdrImage()
-{
-    //qDebug() << "MainWindow::updateActionsHdrImage()";
-
-    m_Ui->fileSaveAsAction->setEnabled(true);
-    m_Ui->actionSave_Hdr_Preview->setEnabled(true);
-    //actionShowHDRs->setEnabled(false);
-
-    // Histogram
-    m_Ui->menuHDR_Histogram->setEnabled(true);
-    m_Ui->Low_dynamic_range->setEnabled(true);
-    m_Ui->Fit_to_dynamic_range->setEnabled(true);
-    m_Ui->Shrink_dynamic_range->setEnabled(true);
-    m_Ui->Extend_dynamic_range->setEnabled(true);
-    m_Ui->Decrease_exposure->setEnabled(true);
-    m_Ui->Increase_exposure->setEnabled(true);
-
-    m_Ui->actionResizeHDR->setEnabled(true);
-    m_Ui->action_Projective_Transformation->setEnabled(true);
-
-    if (tm_status.curr_tm_frame)
-    {
-        if (!tm_status.curr_tm_frame->hasSelection())
-        {
-            m_Ui->cropToSelectionAction->setEnabled(false);
-            m_Ui->removeSelectionAction->setEnabled(false);
-        }
-        else
-        {
-            m_Ui->cropToSelectionAction->setEnabled(true);
-            m_Ui->removeSelectionAction->setEnabled(true);
-        }
-    }
-    m_Ui->rotateccw->setEnabled(true);
-    m_Ui->rotatecw->setEnabled(true);
-    m_Ui->actionFix_Histogram->setEnabled(false);
-}
-
 void MainWindow::updateActions( int w )
 {
-    //qDebug() << "MainWindow::updateActions(" << w << ")";
-    updatePreviousNextActions();
-    if ( w < 0 )
-    {
-        // something wrong happened?
-        updateActionsNoImage();
-    }
-    else
-    {
-        GenericViewer* g_v = (GenericViewer*)m_tabwidget->widget(w);
-        updateMagnificationButtons(g_v);
-        if ( g_v->isHDR() )
-        {
-            // current selected frame is an HDR
-            updateActionsHdrImage();
-        }
-        else
-        {
-            // current selected frame is not an HDR
-            updateActionsLdrImage();
-        }
-    }
+    qDebug() << "MainWindow::updateActions(" << w << ")";
+	bool hasImage = w >= 0;
+	GenericViewer* g_v = hasImage ? (GenericViewer*)m_tabwidget->widget(w) : 0;
+	bool isHdr = g_v ? g_v->isHDR() : false;
+	bool isLdr = g_v ? !g_v->isHDR() : false;
+	LuminanceOptions luminance_opts;
+	bool hasPrinterProfile = !luminance_opts.getPrinterProfileFileName().isEmpty();
+
+    updateMagnificationButtons(g_v); // g_v ? g_v : 0
+
+    m_Ui->fileSaveAsAction->setEnabled(hasImage);
+    m_Ui->actionSave_Hdr_Preview->setEnabled(hasImage && isHdr);
+    m_Ui->fileSaveAllAction->setEnabled(hasImage && curr_num_ldr_open >= 2);
+
+    // Histogram
+    m_Ui->menuHDR_Histogram->setEnabled(isHdr);
+    m_Ui->Low_dynamic_range->setEnabled(isHdr);
+    m_Ui->Fit_to_dynamic_range->setEnabled(isHdr);
+    m_Ui->Shrink_dynamic_range->setEnabled(isHdr);
+    m_Ui->Extend_dynamic_range->setEnabled(isHdr);
+    m_Ui->Decrease_exposure->setEnabled(isHdr);
+    m_Ui->Increase_exposure->setEnabled(isHdr);
+
+    m_Ui->actionResizeHDR->setEnabled(isHdr);
+    m_Ui->action_Projective_Transformation->setEnabled(isHdr);
+    m_Ui->rotateccw->setEnabled(isHdr);
+    m_Ui->rotatecw->setEnabled(isHdr);
+
+    m_Ui->actionFix_Histogram->setEnabled(isLdr);
+    m_Ui->actionSoft_Proofing->setEnabled(isLdr && hasPrinterProfile);
+    m_Ui->actionGamut_Check->setEnabled(isLdr && hasPrinterProfile);
+
+    bool hasCropping = isHdr && tm_status.curr_tm_frame && tm_status.curr_tm_frame->hasSelection();
+    m_Ui->cropToSelectionAction->setEnabled(hasCropping);
+    m_Ui->removeSelectionAction->setEnabled(hasCropping);
 }
 
-void MainWindow::rotateccw_requested()
+void MainWindow::on_rotateccw_triggered()
 {
     dispatchrotate(false);
 }
 
-void MainWindow::rotatecw_requested()
+void MainWindow::on_rotatecw_triggered()
 {
     dispatchrotate(true);
 }
@@ -788,7 +681,7 @@ void MainWindow::dispatchrotate(bool clockwise)
     m_Ui->rotatecw->setEnabled(true);
 }
 
-void MainWindow::resize_requested()
+void MainWindow::on_actionResizeHDR_triggered()
 {
     if (m_tabwidget->count() <= 0) return;
 
@@ -816,7 +709,7 @@ void MainWindow::resize_requested()
     delete resizedialog;
 }
 
-void MainWindow::projectiveTransf_requested()
+void MainWindow::on_action_Projective_Transformation_triggered()
 {
     if (m_tabwidget->count() <= 0) return;
 
@@ -844,7 +737,7 @@ void MainWindow::projectiveTransf_requested()
     delete projTranfsDialog;
 }
 
-void MainWindow::hdr_decrease_exp()
+void MainWindow::on_Decrease_exposure_triggered()
 {
     if (m_tabwidget->count() <= 0) return;
 
@@ -854,7 +747,7 @@ void MainWindow::hdr_decrease_exp()
         curr_hdr_v->lumRange()->decreaseExposure();
 }
 
-void MainWindow::hdr_extend_exp()
+void MainWindow::on_Extend_dynamic_range_triggered()
 {
     if (m_tabwidget->count() <= 0) return;
 
@@ -864,7 +757,7 @@ void MainWindow::hdr_extend_exp()
         curr_hdr_v->lumRange()->extendRange();
 }
 
-void MainWindow::hdr_fit_exp()
+void MainWindow::on_Fit_to_dynamic_range_triggered()
 {
     if (m_tabwidget->count() <= 0) return;
 
@@ -874,7 +767,7 @@ void MainWindow::hdr_fit_exp()
         curr_hdr_v->lumRange()->fitToDynamicRange();
 }
 
-void MainWindow::hdr_increase_exp()
+void MainWindow::on_Increase_exposure_triggered()
 {
     if (m_tabwidget->count() <= 0) return;
 
@@ -884,7 +777,7 @@ void MainWindow::hdr_increase_exp()
         curr_hdr_v->lumRange()->increaseExposure();
 }
 
-void MainWindow::hdr_shrink_exp()
+void MainWindow::on_Shrink_dynamic_range_triggered()
 {
     if (m_tabwidget->count() <= 0) return;
 
@@ -894,7 +787,7 @@ void MainWindow::hdr_shrink_exp()
         curr_hdr_v->lumRange()->shrinkRange();
 }
 
-void MainWindow::hdr_ldr_exp()
+void MainWindow::on_Low_dynamic_range_triggered()
 {
     if (m_tabwidget->count() <= 0) return;
 
@@ -905,7 +798,7 @@ void MainWindow::hdr_ldr_exp()
 }
 
 // Zoom = Viewers (START)
-void MainWindow::viewerZoomIn()
+void MainWindow::on_zoomInAct_triggered()
 {
     if (m_tabwidget->count() <= 0) return;
 
@@ -915,7 +808,7 @@ void MainWindow::viewerZoomIn()
     //updateMagnificationButtons(g_v);
 }
 
-void MainWindow::viewerZoomOut()
+void MainWindow::on_zoomOutAct_triggered()
 {
     if (m_tabwidget->count() <= 0) return;
 
@@ -925,7 +818,7 @@ void MainWindow::viewerZoomOut()
     //updateMagnificationButtons(g_v);
 }
 
-void MainWindow::viewerFitToWin(bool checked)
+void MainWindow::on_fitToWindowAct_triggered()
 {
     if (m_tabwidget->count() <= 0) return;
 
@@ -935,7 +828,7 @@ void MainWindow::viewerFitToWin(bool checked)
     //updateMagnificationButtons(g_v);
 }
 
-void MainWindow::viewerFillToWin()
+void MainWindow::on_actionFill_to_Window_triggered()
 {
     if (m_tabwidget->count() <= 0) return;
 
@@ -945,7 +838,7 @@ void MainWindow::viewerFillToWin()
     //updateMagnificationButtons(g_v);
 }
 
-void MainWindow::viewerOriginalSize()
+void MainWindow::on_normalSizeAct_triggered()
 {
     if (m_tabwidget->count() <= 0) return;
 
@@ -957,7 +850,7 @@ void MainWindow::viewerOriginalSize()
 // Zoom = Viewers (END)
 
 
-void MainWindow::openDocumentation()
+void MainWindow::on_documentationAction_triggered()
 {
     helpBrowser = new HelpBrowser(this,"Luminance HDR Help");
     helpBrowser->setAttribute(Qt::WA_DeleteOnClose);
@@ -991,6 +884,7 @@ void MainWindow::setupIO()
     // progress bar
     m_ProgressBar = new QProgressBar(this);
     m_ProgressBar->hide();
+    statusBar()->addWidget(m_ProgressBar);
 
     // Init Object/Thread
     m_IOThread = new QThread;
@@ -1032,14 +926,15 @@ void MainWindow::ioBegin()
 
     QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
 
-    statusBar()->addWidget(m_ProgressBar);
+    //  statusBar()->addWidget(m_ProgressBar);
     m_ProgressBar->setMaximum(0);
     m_ProgressBar->show();
 }
 
 void MainWindow::ioEnd()
 {
-    statusBar()->removeWidget(m_ProgressBar);
+    //statusBar()->removeWidget(m_ProgressBar);
+    m_ProgressBar->hide();
     m_ProgressBar->reset();
 
     QApplication::restoreOverrideCursor();
@@ -1049,8 +944,13 @@ void MainWindow::ioEnd()
 
 void MainWindow::load_failed(QString error_message)
 {
+    QApplication::restoreOverrideCursor();
+
     // TODO: use unified style?
     QMessageBox::critical(this, tr("Aborting..."), error_message, QMessageBox::Ok, QMessageBox::NoButton);
+
+    m_ProgressBar->hide();
+    m_ProgressBar->reset();
 }
 
 void MainWindow::load_success(pfs::Frame* new_hdr_frame, QString new_fname, bool needSaving)
@@ -1067,7 +967,7 @@ void MainWindow::load_success(pfs::Frame* new_hdr_frame, QString new_fname, bool
         qDebug() << "Filename: " << new_fname;
 #endif
 
-        HdrViewer* newhdr = new HdrViewer(new_hdr_frame, this, false, luminance_options.getViewerNegColor(), luminance_options.getViewerNanInfColor());
+        HdrViewer* newhdr = new HdrViewer(new_hdr_frame, this, needSaving, luminance_options.getViewerNegColor(), luminance_options.getViewerNanInfColor());
 
         newhdr->setAttribute(Qt::WA_DeleteOnClose);
 
@@ -1110,7 +1010,7 @@ void MainWindow::load_success(pfs::Frame* new_hdr_frame, QString new_fname, bool
     }
 }
 
-void MainWindow::preferences_called()
+void MainWindow::on_OptionsAction_triggered()
 {
     unsigned int negcol = luminance_options.getViewerNegColor();
     unsigned int naninfcol = luminance_options.getViewerNanInfColor();
@@ -1134,7 +1034,7 @@ void MainWindow::preferences_called()
     }
 }
 
-void MainWindow::transplant_called()
+void MainWindow::on_Transplant_Exif_Data_action_triggered()
 {
     TransplantExifDialog *transplant=new TransplantExifDialog(this);
     transplant->setAttribute(Qt::WA_DeleteOnClose);
@@ -1159,14 +1059,11 @@ void MainWindow::openFiles(const QStringList& files)
 {
     if (files.size() > 0)
     {
-        DnDOptionDialog dndOption(this, files);
-        dndOption.exec();
-
-        switch (dndOption.result) {
-        case 1: // create new using LDRS
-            fileNewViaWizard(files);
+        switch (DnDOptionDialog::showDndDialog(this, files)) {
+        case DnDOptionDialog::ACTION_NEW_HDR:
+        	createNewHdr(files);
             break;
-        case 2: // open HDRs
+        case DnDOptionDialog::ACTION_OPEN_HDR:
             foreach (QString filename, files)
             {
                  //qDebug() << filename;
@@ -1219,7 +1116,7 @@ void MainWindow::showSplash()
 
 void MainWindow::splashShowDonationsPage()
 {
-    showDonationsPage();
+	on_actionDonate_triggered();
     splash->close();
 }
 
@@ -1229,9 +1126,9 @@ void MainWindow::splashClose()
     splash->close();
 }
 
-void MainWindow::aboutLuminance()
+void MainWindow::on_actionAbout_Luminance_triggered()
 {
-    UMessageBox::about();
+	UMessageBox::about();
 }
 
 /*
@@ -1279,17 +1176,7 @@ void MainWindow::setActiveMainWindow(QWidget* w)
     return;
 }
 
-void MainWindow::minimizeMW()
-{
-    this->showMinimized();
-}
-
-void MainWindow::maximizeMW()
-{
-    this->showMaximized();
-}
-
-void MainWindow::bringAllMWToFront()
+void MainWindow::on_actionBring_All_to_Front_triggered()
 {
     foreach (QWidget *widget, QApplication::topLevelWidgets())
     {
@@ -1301,14 +1188,14 @@ void MainWindow::bringAllMWToFront()
     }
 }
 
-void MainWindow::batch_hdr_requested()
+void MainWindow::on_actionBatch_HDR_triggered()
 {
     BatchHDRDialog *batch_hdr_dialog = new BatchHDRDialog(this);
     batch_hdr_dialog->exec();
     delete batch_hdr_dialog;
 }
 
-void MainWindow::batch_requested()
+void MainWindow::on_actionBatch_Tone_Mapping_triggered()
 {
     BatchTMDialog *batchdialog = new BatchTMDialog(this);
     batchdialog->exec();
@@ -1435,6 +1322,9 @@ void MainWindow::setupTM()
     tmPanel->setEnabled(false);
 
     m_TMProgressBar = new TMOProgressIndicator;
+    m_TMProgressBar->hide();
+    statusBar()->addWidget(m_TMProgressBar);
+
     connect(this, SIGNAL(destroyed()), m_TMProgressBar, SLOT(deleteLater()));
 
     m_TMWorker = new TMWorker;
@@ -1468,14 +1358,15 @@ void MainWindow::setupTM()
 
 void MainWindow::tonemapBegin()
 {
-    statusBar()->addWidget(m_TMProgressBar);
+    // statusBar()->addWidget(m_TMProgressBar);
     m_TMProgressBar->setMaximum(0);
     m_TMProgressBar->show();
 }
 
 void MainWindow::tonemapEnd()
 {
-    statusBar()->removeWidget(m_TMProgressBar);
+    // statusBar()->removeWidget(m_TMProgressBar);
+    m_TMProgressBar->hide();
     m_TMProgressBar->reset();
 }
 
@@ -1485,16 +1376,27 @@ void MainWindow::tonemapImage(TonemappingOptions *opts)
     qDebug() << "Start Tone Mapping";
 #endif
 
-    if ( opts->tmoperator == fattal && luminance_options.isShowFattalWarning() )
+    // Warning when using size dependent TMOs with smaller sizes
+    if ( (opts->tmoperator == fattal) &&
+         (opts->operator_options.fattaloptions.fftsolver == false) &&
+         (opts->xsize != opts->origxsize) &&
+         (luminance_options.isShowFattalWarning()) )
     {
-        // Warning when using size dependent TMOs with smaller sizes
-        if ( opts->xsize != opts->origxsize )
+        TonemappingWarningDialog tonemappingWarningDialog(this);
+
+        switch ( tonemappingWarningDialog.exec() )
         {
-            TonemappingWarningDialog tonemapping_warning_dialog(this);
-
-            tonemapping_warning_dialog.exec();
-
-            if ( !tonemapping_warning_dialog.wasAccepted() ) return;
+        case QMessageBox::Yes :
+        {} break;
+        case QMessageBox::YesAll :
+        {
+            luminance_options.setShowFattalWarning(false);
+        } break;
+        case QMessageBox::No :
+        default:
+        {
+            return;
+        }
         }
     }
 
@@ -1555,6 +1457,15 @@ void MainWindow::addLdrFrame(pfs::Frame *frame, TonemappingOptions* tm_options)
     m_tabwidget->setCurrentWidget(n);
 
     previewPanel->setEnabled(true);
+	
+	if (m_Ui->actionSoft_Proofing->isChecked()) {
+		LdrViewer *viewer = static_cast<LdrViewer *>(n);
+		viewer->doSoftProofing(false);
+	}
+	else if (m_Ui->actionGamut_Check->isChecked()) {
+		LdrViewer *viewer = static_cast<LdrViewer *>(n);
+		viewer->doSoftProofing(true);
+	}
 }
 
 void MainWindow::tonemapFailed(QString error_msg)
@@ -1568,7 +1479,7 @@ void MainWindow::tonemapFailed(QString error_msg)
 /*
  * Lock Handling
  */
-void MainWindow::lockViewers(bool /*toggled*/)
+void MainWindow::on_actionLock_toggled(bool /*toggled*/)
 {
     if (m_Ui->actionLock->isChecked() && m_tabwidget->count())
     {
@@ -1625,72 +1536,23 @@ void MainWindow::showPreviewPanel(bool b)
 
 void MainWindow::updateMagnificationButtons(GenericViewer* c_v)
 {
-    if ( c_v == NULL )
-    {
-        m_Ui->normalSizeAct->setEnabled(false);
-        m_Ui->normalSizeAct->setChecked(false);
-        m_Ui->fitToWindowAct->setEnabled( false );
-        m_Ui->fitToWindowAct->setChecked(false);
-        m_Ui->actionFill_to_Window->setEnabled(false);
-        m_Ui->actionFill_to_Window->setChecked(false);
+	bool hasImage = c_v != NULL;
+	bool isNormalSize = c_v && c_v->isNormalSize();
+	bool isFilledToWindow = c_v && c_v->isFilledToWindow();
+	bool isFittedToWindow = c_v && c_v->isFittedToWindow();
 
-        m_Ui->zoomInAct->setEnabled( false );
-        m_Ui->zoomOutAct->setEnabled( false );
-    }
-    else
-    {  
-        if ( c_v->isNormalSize() )
-        {
-            m_Ui->zoomInAct->setEnabled(false);
-            m_Ui->zoomOutAct->setEnabled(true);
+    m_Ui->zoomInAct->setEnabled(hasImage && (isFilledToWindow || isFittedToWindow));
+    m_Ui->zoomOutAct->setEnabled(hasImage && !isFittedToWindow);
 
-            m_Ui->normalSizeAct->setEnabled(false);
-            m_Ui->normalSizeAct->setChecked(true);
-            m_Ui->fitToWindowAct->setEnabled(true);
-            m_Ui->fitToWindowAct->setChecked(false);
-            m_Ui->actionFill_to_Window->setEnabled(true);
-            m_Ui->actionFill_to_Window->setChecked(false);
-
-            return;
-        }
-
-        if ( c_v->isFilledToWindow() )
-        {
-            m_Ui->zoomInAct->setEnabled(true);
-            m_Ui->zoomOutAct->setEnabled(true);
-
-            m_Ui->normalSizeAct->setEnabled(true);
-            m_Ui->normalSizeAct->setChecked(false);
-            m_Ui->fitToWindowAct->setEnabled(true);
-            m_Ui->fitToWindowAct->setChecked(false);
-            m_Ui->actionFill_to_Window->setEnabled(false);
-            m_Ui->actionFill_to_Window->setChecked(true);
-
-            return;
-        }
-
-        if ( c_v->isFittedToWindow() )
-        {
-            m_Ui->zoomInAct->setEnabled(true);
-            m_Ui->zoomOutAct->setEnabled(false);
-
-            m_Ui->normalSizeAct->setEnabled(true);
-            m_Ui->normalSizeAct->setChecked(false);
-            m_Ui->fitToWindowAct->setEnabled(false);
-            m_Ui->fitToWindowAct->setChecked(true);
-            m_Ui->actionFill_to_Window->setEnabled(true);
-            m_Ui->actionFill_to_Window->setChecked(false);
-
-            return;
-        }
-    }
+    m_Ui->normalSizeAct->setChecked(hasImage && isNormalSize);
+    m_Ui->normalSizeAct->setEnabled(hasImage && !isNormalSize);
+    m_Ui->fitToWindowAct->setChecked(hasImage && isFittedToWindow);
+    m_Ui->fitToWindowAct->setEnabled(hasImage && !isFittedToWindow);
+    m_Ui->actionFill_to_Window->setChecked(hasImage && isFilledToWindow);
+    m_Ui->actionFill_to_Window->setEnabled(hasImage && !isFilledToWindow);
 }
 
-/*
- * Next/Previous Buttons
- */
-
-void MainWindow::removeCurrentTab() 
+void MainWindow::on_actionRemove_Tab_triggered()
 {
 	removeTab(m_tabwidget->currentIndex());
 }
@@ -1705,25 +1567,15 @@ void MainWindow::removeTab(int t)
     w->blockSignals(true);
     if (w->isHDR())
     {
+    	bool doClose = false;
+
         qDebug() << "Remove HDR from MainWindow";
         if ( w->needsSaving() )
         {
             if ( maybeSave() )
             {
                 // if discard OR saved
-                m_tabwidget->removeTab(t);
-                w->deleteLater();   // delete yourself whenever you want
-
-                setWindowModified(false);
-
-                tm_status.is_hdr_ready = false;
-                tm_status.curr_tm_frame = NULL;
-                tm_status.curr_tm_options = NULL;
-
-                tmPanel->setEnabled(false);
-                m_Ui->actionShowPreviewPanel->setEnabled(false);
-
-                previewPanel->hide();
+            	doClose = true;
             }
             // if FALSE, it means that the user said "Cancel"
             // or the saving operation went wrong
@@ -1733,21 +1585,22 @@ void MainWindow::removeTab(int t)
             }
         }
         else
-        {
-            // if discard OR saved
-            m_tabwidget->removeTab(t);
-            w->deleteLater();   // delete yourself whenever you want
+        	doClose = true;
 
-            setWindowModified(false);
+        if (doClose) {
+			m_tabwidget->removeTab(t);
+			w->deleteLater();   // delete yourself whenever you want
 
-            tm_status.is_hdr_ready = false;
-            tm_status.curr_tm_frame = NULL;
-            tm_status.curr_tm_options = NULL;
+			showPreviewPanel(false);
+			setWindowModified(false);
 
-            tmPanel->setEnabled(false);
-            m_Ui->actionShowPreviewPanel->setEnabled(false);
+			tm_status.is_hdr_ready = false;
+			tm_status.curr_tm_frame = NULL;
+			tm_status.curr_tm_options = NULL;
 
-            previewPanel->hide();
+			tmPanel->setEnabled(false);
+
+			previewPanel->hide();
         }
     }
     else
@@ -1759,59 +1612,6 @@ void MainWindow::removeTab(int t)
         if (curr_num_ldr_open == 1)
             m_Ui->fileSaveAllAction->setEnabled(false);
     }
-    updatePreviousNextActions();
-}
-
-void MainWindow::activateNextViewer()
-{
-    int curr_num_viewers = m_tabwidget->count();
-    int curr_viewer = m_tabwidget->currentIndex();
-
-    if ( curr_viewer < curr_num_viewers-1 )
-    {
-        m_tabwidget->setCurrentIndex(curr_viewer+1);
-    }
-}
-
-void MainWindow::activatePreviousViewer()
-{
-    int curr_viewer = m_tabwidget->currentIndex();
-
-    if ( curr_viewer > 0 )
-    {
-        m_tabwidget->setCurrentIndex(curr_viewer-1);
-    }
-}
-
-void MainWindow::updatePreviousNextActions()
-{
-    int curr_num_viewers = m_tabwidget->count();
-    if (curr_num_viewers <= 1)
-    {
-        m_Ui->actionShowNext->setEnabled(false);
-        m_Ui->actionShowPrevious->setEnabled(false);
-        m_Ui->actionLock->setEnabled(false);
-    }
-    else
-    {
-        m_Ui->actionLock->setEnabled(true);
-        int curr_viewer = m_tabwidget->currentIndex();
-        if ( curr_viewer == 0 )
-        {
-            m_Ui->actionShowNext->setEnabled(true);
-            m_Ui->actionShowPrevious->setEnabled(false);
-        }
-        else if (curr_viewer == (curr_num_viewers-1))
-        {
-            m_Ui->actionShowNext->setEnabled(false);
-            m_Ui->actionShowPrevious->setEnabled(true);
-        }
-        else
-        {
-            m_Ui->actionShowNext->setEnabled(true);
-            m_Ui->actionShowPrevious->setEnabled(true);
-        }
-    }
 }
 
 QString MainWindow::getCurrentHDRName()
@@ -1911,7 +1711,7 @@ void MainWindow::clearRecentFileActions()
     }
 }
 
-void MainWindow::levelsRequested(bool checked)
+void MainWindow::on_actionFix_Histogram_toggled(bool checked)
 {
     if (checked)
     {
@@ -1959,3 +1759,66 @@ void MainWindow::openInputFiles()
 {
     openFiles(inputFiles);
 }
+
+void MainWindow::on_actionSoft_Proofing_toggled(bool doProof)
+{
+	GenericViewer* current = (GenericViewer*) m_tabwidget->currentWidget();
+	if ( current==NULL ) return;
+	if ( current->isHDR() ) return;
+	LdrViewer *viewer = (LdrViewer *) current;
+	if (doProof) {
+		qDebug() << "MainWindow:: do soft proofing"; 
+		if (m_Ui->actionGamut_Check->isChecked())
+			m_Ui->actionGamut_Check->setChecked(false);
+		viewer->doSoftProofing(false);
+	}
+	else {
+		qDebug() << "MainWindow:: undo soft proofing";
+		viewer->undoSoftProofing();
+	}
+}
+
+void MainWindow::on_actionGamut_Check_toggled(bool doGamut)
+{
+	GenericViewer* current = (GenericViewer*) m_tabwidget->currentWidget();
+	if ( current==NULL ) return;
+	if ( current->isHDR() ) return;
+	LdrViewer *viewer = (LdrViewer *) current;
+	if (doGamut) {
+		qDebug() << "MainWindow:: do gamut check"; 
+		if (m_Ui->actionSoft_Proofing->isChecked())
+			m_Ui->actionSoft_Proofing->setChecked(false);
+		viewer->doSoftProofing(true);
+	}
+	else {
+		qDebug() << "MainWindow:: undo gamut check"; 
+		viewer->undoSoftProofing();
+	}
+}
+
+#ifdef Q_WS_WIN
+bool MainWindow::winEvent(MSG * message, long * result)
+{
+    return OsIntegration::getInstance().winEvent(message, result);
+}
+#endif
+
+void MainWindow::updateSoftProofing(int i)
+{
+	QWidget *wgt = m_tabwidget->widget(i);
+	GenericViewer *g_v = (GenericViewer *)wgt;
+
+	if (g_v == NULL)
+		return;
+
+	if ( !g_v->isHDR() )
+	{
+		LdrViewer *l_v = static_cast<LdrViewer*>(g_v);
+		if (!m_Ui->actionSoft_Proofing->isChecked() && !m_Ui->actionGamut_Check->isChecked())
+			l_v->undoSoftProofing();
+		else if (m_Ui->actionSoft_Proofing->isChecked())
+			l_v->doSoftProofing(false);
+		else if (m_Ui->actionGamut_Check->isChecked())
+			l_v->doSoftProofing(true);
+	}
+}
