@@ -29,10 +29,14 @@
  * $Id: pde.h,v 1.2 2008/09/04 12:46:49 julians37 Exp $
  */
 
-#ifndef _fmg_pde_h_
-#define _fmg_pde_h_
+#ifndef FMG_PDE_H
+#define FMG_PDE_H
 
-#include "TonemappingOperators/pfstmo.h"
+class ProgressHelper;
+
+namespace pfs {
+    class Array2D;
+}
 
 /**
  * @brief solve pde using full multrigrid algorithm
@@ -40,7 +44,25 @@
  * @param F array with divergence
  * @param U [out] sollution
  */
-void solve_pde_multigrid(pfs::Array2D *F, pfs::Array2D *U);
+void solve_pde_multigrid(pfs::Array2D *F, pfs::Array2D *U, ProgressHelper *ph);
+
+/**
+ * @brief solve poisson pde (Laplace U = F) using discrete cosine transform
+ *
+ * @param F array of the right hand side (contains div G in this example)
+ * @param U [out] solution
+ * @param adjust_bound, adjust boundary values of F to make pde solvable 
+ */
+void solve_pde_fft(pfs::Array2D *F, pfs::Array2D *U, ProgressHelper *ph, bool adjust_bound=false);
+
+/**
+ * @brief returns the residual error of the solution U, ie norm(Laplace U - F) 
+ *
+ * @param F [in] right hand side
+ * @param U [in] solution
+ */
+float residual_pde(pfs::Array2D *U, pfs::Array2D *F);
+
 
 #endif
 
