@@ -45,7 +45,7 @@
 #include "Common/config.h"
 #include "TonemappingPanel/TonemappingPanel.h"
 #include "TonemappingPanel/TMOProgressIndicator.h"
-#include "TonemappingPanel/SavedParametersDialog.h"
+#include "Common/SavedParametersDialog.h"
 #include "TonemappingPanel/SavingParametersDialog.h"
 #include "TonemappingOperators/pfstmdefaultparams.h"
 #include "UI/Gang.h"
@@ -79,15 +79,16 @@ TonemappingPanel::TonemappingPanel(QWidget *parent):
     luminanceLevelGang = new Gang(m_Ui->luminanceLevelSlider, m_Ui->luminanceLevelDSB, m_Ui->luminanceLevelCheckBox, NULL, NULL, NULL, 1.f, 100.0f, MANTIUK08_LUMINANCE_LEVEL);
 
     // fattal02
-    alphaGang = new Gang(m_Ui->alphaSlider, m_Ui->alphadsb, NULL,NULL,NULL,NULL, 1e-4, 2.f, FATTAL02_ALPHA, true);
+    alphaGang = new Gang(m_Ui->alphaSlider, m_Ui->alphadsb, NULL,NULL,NULL,NULL, 0.05, 2.f, FATTAL02_ALPHA, true);
 
     connect(alphaGang, SIGNAL(enableUndo(bool)), m_Ui->undoButton, SLOT(setEnabled(bool)));
     connect(alphaGang, SIGNAL(enableRedo(bool)), m_Ui->redoButton, SLOT(setEnabled(bool)));
 
     betaGang = new Gang(m_Ui->betaSlider, m_Ui->betadsb, NULL,NULL,NULL,NULL, 0.1f, 2.f, FATTAL02_BETA);
-    saturation2Gang = new Gang(m_Ui->saturation2Slider, m_Ui->saturation2dsb, NULL,NULL,NULL,NULL, 0.f, 1.f, FATTAL02_COLOR);
+    saturation2Gang = new Gang(m_Ui->saturation2Slider, m_Ui->saturation2dsb, NULL,NULL,NULL,NULL, 0.f, 1.5f, FATTAL02_COLOR);
     noiseGang = new Gang(m_Ui->noiseSlider, m_Ui->noisedsb, NULL,NULL,NULL,NULL, 0, 1.f, FATTAL02_NOISE_REDUX);
-    oldFattalGang = new Gang(NULL,NULL, m_Ui->oldFattalCheckBox);
+    // oldFattalGang = new Gang(NULL,NULL, m_Ui->oldFattalCheckBox);
+    fftSolverGang = new Gang(NULL, NULL,m_Ui->fftVersionCheckBox);
 
     // ashikhmin02
     contrastGang = new Gang(m_Ui->contrastSlider, m_Ui->contrastdsb,NULL,NULL,NULL,NULL, 0.f, 1.f, 0.5f);
@@ -195,6 +196,8 @@ TonemappingPanel::~TonemappingPanel()
     delete lightGang;
     delete pregammaGang;
 
+	qDeleteAll(toneMappingOptionsToDelete);
+
 	QSqlDatabase db = QSqlDatabase::database();
 	db.close();
 }
@@ -313,7 +316,6 @@ void TonemappingPanel::on_defaultButton_clicked()
         betaGang->setDefault();
         saturation2Gang->setDefault();
         noiseGang->setDefault();
-        m_Ui->oldFattalCheckBox->setChecked(false);
         break;
     case mantiuk06:
         contrastfactorGang->setDefault();
@@ -402,77 +404,80 @@ void TonemappingPanel::on_applyButton_clicked()
     fillToneMappingOptions();
     setupUndo();
 
-    emit startTonemapping(&toneMappingOptions);
+    emit startTonemapping(toneMappingOptions);
 }
 
 void TonemappingPanel::fillToneMappingOptions()
-{
-    toneMappingOptions.origxsize = sizes[0];
-    toneMappingOptions.xsize = sizes[m_Ui->sizeComboBox->currentIndex()];
-    toneMappingOptions.pregamma = pregammaGang->v();
-    //toneMappingOptions.tonemapSelection = checkBoxSelection->isChecked();
-    //toneMappingOptions.tonemapOriginal = checkBoxOriginal->isChecked();
+{	
+	toneMappingOptions = new TonemappingOptions;
+	toneMappingOptionsToDelete.push_back(toneMappingOptions);
+    toneMappingOptions->origxsize = sizes[0];
+    toneMappingOptions->xsize = sizes[m_Ui->sizeComboBox->currentIndex()];
+    toneMappingOptions->pregamma = pregammaGang->v();
+    // toneMappingOptions->tonemapSelection = checkBoxSelection->isChecked();
+    // toneMappingOptions->tonemapOriginal = checkBoxOriginal->isChecked();
     switch (currentTmoOperator) {
     case ashikhmin:
-        toneMappingOptions.tmoperator = ashikhmin;
-        toneMappingOptions.operator_options.ashikhminoptions.simple=simpleGang->isCheckBox1Checked();
-        toneMappingOptions.operator_options.ashikhminoptions.eq2=eq2Gang->isRadioButton1Checked();
-        toneMappingOptions.operator_options.ashikhminoptions.lct=contrastGang->v();
+        toneMappingOptions->tmoperator = ashikhmin;
+        toneMappingOptions->operator_options.ashikhminoptions.simple=simpleGang->isCheckBox1Checked();
+        toneMappingOptions->operator_options.ashikhminoptions.eq2=eq2Gang->isRadioButton1Checked();
+        toneMappingOptions->operator_options.ashikhminoptions.lct=contrastGang->v();
         break;
     case drago:
-        toneMappingOptions.tmoperator = drago;
-        toneMappingOptions.operator_options.dragooptions.bias=biasGang->v();
+        toneMappingOptions->tmoperator = drago;
+        toneMappingOptions->operator_options.dragooptions.bias=biasGang->v();
         break;
     case durand:
-        toneMappingOptions.tmoperator = durand;
-        toneMappingOptions.operator_options.durandoptions.spatial=spatialGang->v();
-        toneMappingOptions.operator_options.durandoptions.range=rangeGang->v();
-        toneMappingOptions.operator_options.durandoptions.base=baseGang->v();
+        toneMappingOptions->tmoperator = durand;
+        toneMappingOptions->operator_options.durandoptions.spatial=spatialGang->v();
+        toneMappingOptions->operator_options.durandoptions.range=rangeGang->v();
+        toneMappingOptions->operator_options.durandoptions.base=baseGang->v();
         break;
     case fattal:
-        toneMappingOptions.tmoperator = fattal;
-        toneMappingOptions.operator_options.fattaloptions.alpha=alphaGang->v();
-        toneMappingOptions.operator_options.fattaloptions.beta=betaGang->v();
-        toneMappingOptions.operator_options.fattaloptions.color=saturation2Gang->v();
-        toneMappingOptions.operator_options.fattaloptions.noiseredux=noiseGang->v();
-        toneMappingOptions.operator_options.fattaloptions.newfattal=!oldFattalGang->isCheckBox1Checked();
+        toneMappingOptions->tmoperator = fattal;
+        toneMappingOptions->operator_options.fattaloptions.alpha=alphaGang->v();
+        toneMappingOptions->operator_options.fattaloptions.beta=betaGang->v();
+        toneMappingOptions->operator_options.fattaloptions.color=saturation2Gang->v();
+        toneMappingOptions->operator_options.fattaloptions.noiseredux=noiseGang->v();
+//        toneMappingOptions->operator_options.fattaloptions.newfattal=!oldFattalGang->isCheckBox1Checked();
+        toneMappingOptions->operator_options.fattaloptions.fftsolver=fftSolverGang->isCheckBox1Checked();
         break;
     case mantiuk06:
-        toneMappingOptions.tmoperator = mantiuk06;
-        toneMappingOptions.operator_options.mantiuk06options.contrastfactor=contrastfactorGang->v();
-        toneMappingOptions.operator_options.mantiuk06options.saturationfactor=saturationfactorGang->v();
-        toneMappingOptions.operator_options.mantiuk06options.detailfactor=detailfactorGang->v();
-        toneMappingOptions.operator_options.mantiuk06options.contrastequalization=contrastfactorGang->isCheckBox1Checked();
+        toneMappingOptions->tmoperator = mantiuk06;
+        toneMappingOptions->operator_options.mantiuk06options.contrastfactor=contrastfactorGang->v();
+        toneMappingOptions->operator_options.mantiuk06options.saturationfactor=saturationfactorGang->v();
+        toneMappingOptions->operator_options.mantiuk06options.detailfactor=detailfactorGang->v();
+        toneMappingOptions->operator_options.mantiuk06options.contrastequalization=contrastfactorGang->isCheckBox1Checked();
         break;
     case mantiuk08:
-        toneMappingOptions.tmoperator = mantiuk08;
-        toneMappingOptions.operator_options.mantiuk08options.colorsaturation=colorSaturationGang->v();
-        toneMappingOptions.operator_options.mantiuk08options.contrastenhancement=contrastEnhancementGang->v();
-        toneMappingOptions.operator_options.mantiuk08options.luminancelevel=luminanceLevelGang->v();
-        toneMappingOptions.operator_options.mantiuk08options.setluminance=luminanceLevelGang->isCheckBox1Checked();
+        toneMappingOptions->tmoperator = mantiuk08;
+        toneMappingOptions->operator_options.mantiuk08options.colorsaturation=colorSaturationGang->v();
+        toneMappingOptions->operator_options.mantiuk08options.contrastenhancement=contrastEnhancementGang->v();
+        toneMappingOptions->operator_options.mantiuk08options.luminancelevel=luminanceLevelGang->v();
+        toneMappingOptions->operator_options.mantiuk08options.setluminance=luminanceLevelGang->isCheckBox1Checked();
         break;
     case pattanaik:
-        toneMappingOptions.tmoperator = pattanaik;
-        toneMappingOptions.operator_options.pattanaikoptions.autolum=autoYGang->isCheckBox1Checked();
-        toneMappingOptions.operator_options.pattanaikoptions.local=pattalocalGang->isCheckBox1Checked();
-        toneMappingOptions.operator_options.pattanaikoptions.cone=coneGang->v();
-        toneMappingOptions.operator_options.pattanaikoptions.rod=rodGang->v();
-        toneMappingOptions.operator_options.pattanaikoptions.multiplier=multiplierGang->v();
+        toneMappingOptions->tmoperator = pattanaik;
+        toneMappingOptions->operator_options.pattanaikoptions.autolum=autoYGang->isCheckBox1Checked();
+        toneMappingOptions->operator_options.pattanaikoptions.local=pattalocalGang->isCheckBox1Checked();
+        toneMappingOptions->operator_options.pattanaikoptions.cone=coneGang->v();
+        toneMappingOptions->operator_options.pattanaikoptions.rod=rodGang->v();
+        toneMappingOptions->operator_options.pattanaikoptions.multiplier=multiplierGang->v();
         break;
     case reinhard02:
-        toneMappingOptions.tmoperator = reinhard02;
-        toneMappingOptions.operator_options.reinhard02options.scales=usescalesGang->isCheckBox1Checked();
-        toneMappingOptions.operator_options.reinhard02options.key=keyGang->v();
-        toneMappingOptions.operator_options.reinhard02options.phi=phiGang->v();
-        toneMappingOptions.operator_options.reinhard02options.range=(int)range2Gang->v();
-        toneMappingOptions.operator_options.reinhard02options.lower=(int)lowerGang->v();
-        toneMappingOptions.operator_options.reinhard02options.upper=(int)upperGang->v();
+        toneMappingOptions->tmoperator = reinhard02;
+        toneMappingOptions->operator_options.reinhard02options.scales=usescalesGang->isCheckBox1Checked();
+        toneMappingOptions->operator_options.reinhard02options.key=keyGang->v();
+        toneMappingOptions->operator_options.reinhard02options.phi=phiGang->v();
+        toneMappingOptions->operator_options.reinhard02options.range=(int)range2Gang->v();
+        toneMappingOptions->operator_options.reinhard02options.lower=(int)lowerGang->v();
+        toneMappingOptions->operator_options.reinhard02options.upper=(int)upperGang->v();
         break;
     case reinhard05:
-        toneMappingOptions.tmoperator = reinhard05;
-        toneMappingOptions.operator_options.reinhard05options.brightness=brightnessGang->v();
-        toneMappingOptions.operator_options.reinhard05options.chromaticAdaptation=chromaticGang->v();
-        toneMappingOptions.operator_options.reinhard05options.lightAdaptation=lightGang->v();
+        toneMappingOptions->tmoperator = reinhard05;
+        toneMappingOptions->operator_options.reinhard05options.brightness=brightnessGang->v();
+        toneMappingOptions->operator_options.reinhard05options.chromaticAdaptation=chromaticGang->v();
+        toneMappingOptions->operator_options.reinhard05options.lightAdaptation=lightGang->v();
         break;
     }
 }
@@ -499,7 +504,8 @@ void TonemappingPanel::setupUndo()
         betaGang->setupUndo();
         saturation2Gang->setupUndo();
         noiseGang->setupUndo();
-        oldFattalGang->setupUndo();
+//        oldFattalGang->setupUndo();
+        fftSolverGang->setupUndo();
         break;
     case mantiuk06:
         contrastfactorGang->setupUndo();
@@ -556,7 +562,8 @@ void TonemappingPanel::on_undoButton_clicked()
         betaGang->undo();
         saturation2Gang->undo();
         noiseGang->undo();
-        oldFattalGang->undo();
+//        oldFattalGang->undo();
+        fftSolverGang->undo();
         break;
     case mantiuk06:
         contrastfactorGang->undo();
@@ -613,7 +620,8 @@ void TonemappingPanel::on_redoButton_clicked()
         betaGang->redo();
         saturation2Gang->redo();
         noiseGang->redo();
-        oldFattalGang->redo();
+//        oldFattalGang->redo();
+        fftSolverGang->redo();
         break;
     case mantiuk06:
         contrastfactorGang->redo();
@@ -713,7 +721,6 @@ void TonemappingPanel::fromGui2Txt(QString destination)
     out << "# Please edit this file by hand only if you know what you're doing, in any case never change the left hand side text (i.e. the part before the ``='')." << endl;
     out << "TMOSETTINGSVERSION=" << TMOSETTINGSVERSION << endl;
     out << "XSIZE=" << sizes[m_Ui->sizeComboBox->currentIndex()] << endl;
-    out << "QUALITY=" << LuminanceOptions().getBatchTmDefaultOutputQuality() << endl;
 
     QWidget *current_page = m_Ui->stackedWidget_operators->currentWidget();
     if (current_page == m_Ui->page_mantiuk06)
@@ -739,7 +746,7 @@ void TonemappingPanel::fromGui2Txt(QString destination)
         out << "BETA=" << betaGang->v() << endl;
         out << "COLOR=" << saturation2Gang->v() << endl;
         out << "NOISE=" << noiseGang->v() << endl;
-        out << "OLDFATTAL=" << (m_Ui->oldFattalCheckBox->isChecked() ? "YES" : "NO") << endl;
+        out << "OLDFATTAL=NO" << endl;
     }
     else if (current_page == m_Ui->page_ashikhmin)
     {
@@ -900,8 +907,8 @@ void TonemappingPanel::fromTxt2Gui()
             m_Ui->saturation2Slider->setValue(saturation2Gang->v2p(value.toFloat()));
         } else if (field == "NOISE") {
             m_Ui->noiseSlider->setValue(noiseGang->v2p(value.toFloat()));
-        } else if (field == "OLDFATTAL") {
-            m_Ui->oldFattalCheckBox->setChecked(value == "YES");
+//        } else if (field == "OLDFATTAL") {
+//            m_Ui->oldFattalCheckBox->setChecked(value == "YES");
         } else if (field == "MULTIPLIER") {
             m_Ui->multiplierSlider->setValue(multiplierGang->v2p(value.toFloat()));
         } else if (field == "LOCAL") {
@@ -1098,7 +1105,7 @@ void TonemappingPanel::saveParameters()
 		        beta = betaGang->v();
         		colorSat = saturation2Gang->v();
 		        noiseReduction = noiseGang->v();
-        		oldFattal = oldFattalGang->isCheckBox1Checked();
+                oldFattal = false; //always false! // oldFattalGang->isCheckBox1Checked();
 				execFattalQuery(alpha, beta, colorSat, noiseReduction, oldFattal, comment);
 	        break;
     		case mantiuk06:
@@ -1144,10 +1151,10 @@ void TonemappingPanel::saveParameters()
 
 void TonemappingPanel::loadParameters()
 {
-	SavedParameters dialog(currentTmoOperator);
+	SavedParametersDialog dialog(currentTmoOperator);
 	if (dialog.exec())
 	{
-		QSqlTableModel *model = dialog.getModel();
+		QSqlQueryModel *model = dialog.getModel();
 		if (model->rowCount() == 0)
 			return;
 		int selectedRow = dialog.getCurrentIndex().row();
@@ -1250,7 +1257,7 @@ void TonemappingPanel::loadParameters()
                 m_Ui->saturation2dsb->setValue(colorSat);
                 m_Ui->noiseSlider->setValue(noiseReduction);
                 m_Ui->noisedsb->setValue(noiseReduction);
-                m_Ui->oldFattalCheckBox->setChecked(oldFattal);
+//                m_Ui->oldFattalCheckBox->setChecked(oldFattal);
                 m_Ui->pregammaSlider->setValue(pregamma);
                 m_Ui->pregammadsb->setValue(pregamma);
 			break;
@@ -1346,10 +1353,10 @@ void TonemappingPanel::loadParameters()
 
 void TonemappingPanel::loadComments()
 {
-    SavedParameters dialog(this);
+    SavedParametersDialog dialog(this);
 	if (dialog.exec())
 	{
-		QSqlTableModel *model = dialog.getModel();
+		QSqlQueryModel *model = dialog.getModel();
 		int selectedRow = dialog.getCurrentIndex().row();
 		QString comment, tmOperator;
 		comment = model->record(selectedRow).value("comment").toString();
@@ -1418,7 +1425,7 @@ void TonemappingPanel::loadComments()
                 m_Ui->saturation2dsb->setValue(query.value(2).toFloat());
                 m_Ui->noiseSlider->setValue(query.value(3).toFloat());
                 m_Ui->noisedsb->setValue(query.value(3).toFloat());
-                m_Ui->oldFattalCheckBox->setChecked(query.value(4).toBool());
+//                m_Ui->oldFattalCheckBox->setChecked(query.value(4).toBool());
                 m_Ui->pregammaSlider->setValue(query.value(5).toFloat());
                 m_Ui->pregammadsb->setValue(query.value(5).toFloat());
 			}
@@ -1470,9 +1477,9 @@ void TonemappingPanel::loadComments()
                 m_Ui->rodSlider->setValue(query.value(2).toFloat());
                 m_Ui->roddsb->setValue(query.value(2).toFloat());
                 m_Ui->pattalocal->setChecked(query.value(3).toBool());
-                m_Ui->autoYcheckbox->setChecked(query.value(3).toFloat());
-                m_Ui->pregammaSlider->setValue(query.value(4).toFloat());
-                m_Ui->pregammadsb->setValue(query.value(4).toFloat());
+                m_Ui->autoYcheckbox->setChecked(query.value(4).toBool());
+                m_Ui->pregammaSlider->setValue(query.value(5).toFloat());
+                m_Ui->pregammadsb->setValue(query.value(5).toFloat());
 			}
 		}
 		else if (tmOperator == "reinhard02")
@@ -1620,7 +1627,7 @@ void TonemappingPanel::execFattalQuery(float alpha, float beta, float colorSatur
 	query.bindValue(":beta", beta);
 	query.bindValue(":colorSaturation", colorSaturation);
 	query.bindValue(":noiseReduction", noiseReduction);
-	query.bindValue(":oldFattal", oldFattal);
+    query.bindValue(":oldFattal", oldFattal);
 	query.bindValue(":pregamma", pregamma);
 	query.bindValue(":comment", comment);
 	bool res = query.exec();
